package org.remote.roadhog;

import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.text.*;

import javax.swing.*;

public abstract class Gui extends javax.swing.JFrame
{

	private boolean debug = true;

	public Gui (String title)
	{
		super (title);
		addWindowListener (new WindowAdapter () { 
			public void windowClosing (WindowEvent e) { quit (); }
		});
	}

	/**
	 * Pops up a file selection dialog and returns the name of the selected file.
	 */
	public File fileSelect ()
	{
		return fileSelect (null, null, false);
	}

	/**
	 * Pops up a file selection dialog and returns the name of the selected file.
	 */
	public File fileSelect (final String type, final String[] extension)
	{
		return fileSelect (type, extension, false);
	}
	
	/**
	 * Pops up a file selection or save dialog and returns the name of the selected file.
	 */
	public File fileSelect (final String type, final String[] extension, boolean save)
	{
		JFileChooser chooser = new JFileChooser (new File (System.getProperty("user.dir")));
		if (type != null)
		{
			chooser.setFileFilter (new javax.swing.filechooser.FileFilter () {
				public String getDescription () { return type; };
				public boolean accept (File f) { 
					if (f.isDirectory ()) return true;
					String name = f.getName ();
					int dot = name.lastIndexOf (".");
					if (dot < 1) return false;
					String ext = name.substring (dot + 1).toLowerCase ();
					for (int i=0;i<extension.length;i++) if(extension[i].equals(ext)) return true;
					return false;
				}
			});
		}
		int returnVal = save ? chooser.showSaveDialog (this) : chooser.showOpenDialog (this);
    	if (returnVal == JFileChooser.APPROVE_OPTION) 
		{
			return chooser.getSelectedFile();
		}
		return null;
	}

	/**
	 * Pops up an error message and waits for confirmation.	
	 */
	public void errorMessage (String msg, Exception e)
	{
		String[] ma;
		if (e != null)
		{
			String c = e.getClass ().getName ();
			int dot = c.lastIndexOf (".");
			ma = new String[] { msg + ":", e.getMessage (), "(" + c.substring (dot + 1) + ")" };
			if (debug) e.printStackTrace ();
		}
		else
		{
			ma = new String[] { msg };
		}
		JOptionPane.showMessageDialog(this.getContentPane (), ma, "Error", 
			JOptionPane.ERROR_MESSAGE);
	}

	public void quit ()
	{
		setVisible (false);
		System.exit (0);
	}

}
