
package org.remote.roadhog;

import java.awt.event.*;
import java.awt.geom.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import java.text.*;
import java.beans.*;

import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;

import org.w3c.dom.*;

import org.apache.xerces.parsers.DOMParser;

class Journey extends XMLObject
{
/*
	String id;
*/
	private HashMap detail = new HashMap ();
	Vertex from;
	Vertex to;
	private String alternative = null;
	Photo[] photo;
	long timeCorrection;
	private String handle;
	private Edge theEdge;
	private Vector waypoints;
	public double priority = 1.0;
	
	public Journey ()
	{
		/* default */
	}

	public Journey (Node node, XMLObjectLookup lookup)
	throws ParseException
	{
		this (node, lookup, true, true);
	}

	public Journey (Node node, XMLObjectLookup lookup, boolean loadWaypoints, boolean loadPhotos)
	throws ParseException
	{
		NodeList sublist;
		Element journey = (Element) node;

		parseAttributeWithLanguage (journey, "detail", detail);

		sublist = journey.getElementsByTagName ("alternative");
		if (sublist.getLength () > 0) alternative = getTextChild (sublist.item(0));

		sublist = journey.getElementsByTagName ("priority");
		if (sublist.getLength () > 0) priority = Double.parseDouble (getTextChild (sublist.item(0)));

		sublist = journey.getElementsByTagName ("from");
		if (sublist.getLength () > 0)
		{
			String fromId = getTextChild (sublist.item(0));
			from = (Vertex) lookup.findObject (Vertex.class, fromId);
			if (from == null)
			{
				throw new ParseException ("Unresolved reference to Vertex id='" + fromId + "'");
			}
		}

		sublist = journey.getElementsByTagName ("to");
		if (sublist.getLength () > 0)
		{
			String toId = getTextChild (sublist.item(0));
			to = (Vertex) lookup.findObject (Vertex.class, toId);
			if (to == null)
			{
				throw new ParseException ("Unresolved reference to Vertex id='" + toId + "'");
			}
		}

		sublist = journey.getElementsByTagName ("photo");
		int length = sublist.getLength ();
		if (length > 0)
		{
			if (!loadPhotos) length = 1;
			photo = new Photo[length];
			for (int i=0; i<length; i++)
			{
				photo[i] = new Photo (sublist.item (i));
				photo[i].selected = true;
			}
		} 
		else
		{
			photo = null;
		}

		sublist = journey.getElementsByTagName ("timecorrection");
		try
		{
			if (sublist.getLength () > 0) 
				timeCorrection = Long.parseLong (getTextChild (sublist.item(0)));
		} 	
		catch (NumberFormatException nf)
		{
			throw new ParseException ("Number format error in timecorrection tag.");
		}

		if (loadWaypoints)
		{
			// slightly different to avoid also receiving second-level objects
			sublist = journey.getChildNodes ();
			for (int i=0; i<sublist.getLength (); i++)
			{
				if ((sublist.item (i).getNodeType () == Node.ELEMENT_NODE) &&
					(sublist.item (i).getNodeName ().equals ("waypoint")))
				{
					addWaypoint (new Waypoint ((Element) sublist.item (i)));
				}
			}
		}
	}

	public String getDetail (String lang)
	{
		return getAttributeInLanguage (detail, lang);
	}

	public void resetWaypoints ()
	{
		if (waypoints != null)
		{
			waypoints.clear ();
		}
	}

	public Vector getWaypoints ()
	{
		return waypoints;
	}

	public void addWaypoint (Waypoint w)
	{
		if (waypoints == null) 
		{
			waypoints = new Vector ();
		}
		waypoints.addElement (w);
	}

	public void toXML (PrintStream o, String indent)
	{
		o.println (indent + "<journey>");
		handle = null; // force recalc
		o.println (indent + "   <id>" + stringToXML (getHandle ()) + "</id>");
		writeAttributeWithLanguage (o, indent + "   ", "detail", detail);

		if (alternative != null) 
			o.println (indent + "   <alternative>" + stringToXML (alternative) + "</alternative>");
		if (priority != 1)
			o.println (indent + "   <priority>"+priority+"</priority>");

		if (from != null)
			o.println (indent + "   <from>" + stringToXML (from.id) + "</from>");
		if (to != null)
			o.println (indent + "   <to>" + stringToXML (to.id) + "</to>");
		if (photo != null)
		{
			for (int i=0; i<photo.length; i++)
			{
				if (photo[i].selected) photo[i].toXML (o, indent + "   ");
			}
		}
		if (waypoints != null)
		{
			for (Enumeration e = waypoints.elements (); e.hasMoreElements (); )
			{
				((Waypoint) e.nextElement ()).toXML (o, indent + "   ");
			}
		}
		o.println (indent + "</journey>");
	}

	/**
	 * The edge (i.e., road) that this journey runs on is not stored inside the
	 * journey object. The journey object only know between which vertices it runs,
	 * and theoretically this is only possible if there also is an edge object.
	 * This function processes a list of edge objects and returns that which 
	 * matches this journey (or null).
	 */
	public Edge findEdge (Iterator i)
	{
		if (theEdge != null)
		{
			return theEdge;
		}
		while (i.hasNext ())
		{
			Edge e = (Edge) i.next ();
			if (edgeMatches (e))
			{
				theEdge = e;
				return e;
			}
		}
		return null;
	}

	/**
	 * Same with Enumeration.
	 */
	public Edge findEdge (Enumeration i)
	{
		if (theEdge != null)
		{
			return theEdge;
		}
		while (i.hasMoreElements ())
		{
			Edge e = (Edge) i.nextElement ();
			if (edgeMatches (e)) 
			{
				theEdge = e;
				return e;
			}
		}
		return null;
	}

	/**
	 * Same with Array. You never know.
	 */
	public Edge findEdge (Edge[] e)
	{
		if (theEdge != null)
		{
			return theEdge;
		}
		for (int i = 0; i < e.length; i++)
		{
			if (edgeMatches (e[i])) 
			{
				theEdge = e[i];
				return e[i];
			}
		}
		return null;
	}

	/**
	 * true if the edge connects the two vertices that this journey runs in between.
	 */
	public boolean edgeMatches (Edge e)
	{
//System.out.print("match "+e.vertex[0]+"/"+from+" "+e.vertex[1]+"/"+to);
		boolean rv = (((e.vertex[0] == from) && (e.vertex[1] == to)) ||
		              ((e.vertex[1] == from) && (e.vertex[0] == to)));
		if (rv && (alternative != null))
		{
			rv = (alternative.equals (e.alternative));
		}
//System.out.println(rv);
		return rv;
	}

	public String getHandle ()
	{
		if (from == null || to == null || photo == null || photo.length < 1)
		{
			return "null";
		}
		if (handle == null)
		{
			MD5 m = new MD5 ();
			String some = from.id + to.id + photo[0].time;
			m.update (some.getBytes ());
			byte[] bytes = new byte[16];
			m.md5final (bytes);
			handle = dumpBytes (bytes).substring (0, 4);
		}
		return handle;
	}

	public Edge getEdge ()
	{
		return theEdge;
	}

	public String getAlternative ()
	{
		return alternative;
	}
			
}
