
package org.remote.roadhog;

import java.io.*;
import java.util.*;

import org.w3c.dom.*;
import org.xml.sax.SAXException;
import org.apache.xerces.parsers.DOMParser;

public class XMLUtils
{

	public static void readFile (DOMParser parser, String filename, final HashMap vertices, 
		final HashMap maps, HashMap edges, HashMap journeys, Vector otherStuff, PrintStream debug, boolean loadWaypoints, boolean loadPhotos)
	throws ParseException, IOException, SAXException
	{

		int vertCount = 0;
		int jourCount = 0;
		int edgeCount = 0;
		int mapCount = 0;

		XMLObject.XMLObjectLookup lookup = new XMLObject.XMLObjectLookup() {
			public Object findObject (Class theClass, String id)
			{
				if (theClass == Vertex.class)
				{
					return vertices.get (id);
				}
				else if (theClass == Map.class)
				{
					return maps.get (id);
				}
				return null;
			}
		};

		if (debug != null) debug.println ("Parsing " + filename + "...");

		parser.parse (filename);

		// run through all the top-level nodes 

		NodeList top = parser.getDocument ().getDocumentElement ().getChildNodes ();

		if (top.getLength () == 0)
		{
			if (debug != null) debug.println ("  no elements present");
		}
		else
		{
			for (int i=0; i < top.getLength (); i++)
			{
				Node child = top.item (i);
				switch (child.getNodeType ())
				{ 
				  case Node.TEXT_NODE:
					/* ignore */
					break;
				  case Node.COMMENT_NODE:
					/* ignore */
					break;
				  case Node.ELEMENT_NODE:
					String name = child.getNodeName ();
					if (name.equals ("vertex"))
					{
						Vertex v = new Vertex (child);
						if (vertices.containsKey (v.id) && (debug != null))
						{
							debug.println ("  redefinition of vertex '" + v.id + "' ignored");
						}
						else
						{
							vertices.put (v.id, v);
							vertCount ++;
						}
					}
					else if (name.equals ("edge"))
					{
						// FIXME check for duplicates
						Edge e = new Edge (child, lookup);
						String id = e.getHandle ();
						if (edges.containsKey (id) && (debug != null))
						{
							debug.println ("  redefinition of edge '" + id + "' ignored");
						}
						else
						{
							edges.put (id, e);
							edgeCount++;
						}
					}
					else if (name.equals ("map"))
					{
						// FIXME check for duplicates
						Map m = new Map (child);
						if (maps.containsKey (m.id) && (debug != null))
						{
							debug.println ("  redefinition of map '" + m.id + "' ignored");
						}
						else
						{
							maps.put (m.id, m);
							mapCount++;
						}
					}
					else if (name.equals ("journey"))
					{
						Journey j = new Journey (child, lookup, loadWaypoints, loadPhotos);
						if (journeys.containsKey (j.getHandle ()) && (debug != null))
						{
							debug.println ("  redefinition of journey '" + j.getHandle () + "' ignored");
						}
						else
						{
							journeys.put (j.getHandle (), j);
							jourCount ++;
						}
					}
					else if (name.equals ("showcase-item"))
					{
						if (otherStuff != null) otherStuff.addElement (new ShowcaseItem (child));
					}
					else if (debug != null)
					{
						debug.println ("  unknown element node '" + name + "' ignored");
					}
					break;
				  default:
					if (debug != null)
						debug.println ("  node '" + child.getNodeName () + "' ignored [" + 
							child.getNodeType () + "]");
					break;
				}
			}
		}
		
		if (debug != null)
			debug.println ("Parsing of " + filename + " finished (" + vertCount + " vertices, " + 
				mapCount + " maps, " + edgeCount + " edges, " + jourCount + " journeys)\n");

	}
}
